;**************************************************************************
;*
;* Boot-ROM-Code to load an operating system across a TCP/IP network.
;*
;* Module:  printf.asm
;* Purpose: Print a formatted string
;* Entries: _printf
;*
;**************************************************************************
;*
;* Copyright (C) 1995,1996 Gero Kuhlmann <gero@gkminix.han.de>
;*
;*  This program is free software; you can redistribute it and/or modify
;*  it under the terms of the GNU General Public License as published by
;*  the Free Software Foundation; either version 2 of the License, or
;*  any later version.
;*
;*  This program is distributed in the hope that it will be useful,
;*  but WITHOUT ANY WARRANTY; without even the implied warranty of
;*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;*  GNU General Public License for more details.
;*
;*  You should have received a copy of the GNU General Public License
;*  along with this program; if not, write to the Free Software
;*  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
;*


;
;**************************************************************************
;
; Include assembler macros:
;
include ..\..\headers\asm\macros.inc
include ..\..\headers\asm\layout.inc
include ..\..\headers\asm\memory.inc
include .\libpriv.inc


;
;**************************************************************************
;
; BSS segment:
;
bss_start

numbuf		db	12 dup (?)	; buffer for number conversion

bss_end


;
;**************************************************************************
;
; Start code segment.
;
text_start

	public	_printf			; define entry points


;
;**************************************************************************
;
; Print formatted string.
; Input:  1. arg  -  format string
;         ...     -  arguments according to format string
; Output: none
;
_printf		proc	near

	penter				; setup standard stack frame
	push	si
	push	di
	getarg	si,0			; get pointer to format string
	mov	bx,1			; BX = argument counter
	cld

; The following is the basic printing routine. It checks, wether there
; is a special character in the format string. If it is not, the current
; character just gets printed out.

print1:	lodsb
print5:	or	al,al			; at end of string?
	jz	short print3
	cmp	al,'%'			; print argument?
	je	short prna1
print2:	cmp	al,CHR_LF		; convert linefeed into newline
	jne	short print4
	int	29h
	mov	al,CHR_CR
print4:	int	29h			; print character
	jmp	short print1		; and check for next character
print3:	jmp	print9			; jump to end of routine

; Handle argument format. This is a bit simpler than usually implemented
; in a C library. If the first character after the initial % is another
; %, print this as a single % character. Otherwise the argument format
; is as follows:
;
;	%[width]indicator
;
; with indicator being one of the following:
;
;	c  -  character
;	s  -  string
;	d  -  decimal number (signed)
;	u  -  decimal number (unsigned)
;	o  -  octal number (unsigned)
;	x  -  unsigned hexadecimal number (unsigned)
;
; The width field can be missing, or be the character 'l', which means a
; long argument for integer numbers, or a pointer to a string followed
; by a length argument for a string argument.

prna1:	lodsb				; get next character
	or	al,al
	jz	short print3
	cmp	al,'%'
	je	short print4		; print % character
	xor	ah,ah
	cmp	al,'l'			; long argument indicator?
	jne	short prna2
	inc	ah
	lodsb
	or	al,al
	jz	short print3

prna2:	mov	dl,10			; check for decimal number
	cmp	al,'d'
	je	short prna3
	cmp	al,'u'
	je	short prna3
	mov	dl,8			; check for octal number
	cmp	al,'o'
	je	short prna3
	mov	dl,16			; check for hexdecimal number
	cmp	al,'x'
	jne	short prna5

prna3:	push	dx
	getarg	cx,bx			; get next argument
	inc	bx
	xor	dx,dx
	cmp	al,'d'			; is it a signed number?
	jne	short prnaA
	xchg	ax,cx
	cwd				; expand word to double word if signed
	xchg	cx,ax
prnaA:	or	ah,ah			; is it a long argument?
	jz	short prna4
	getarg	dx,bx			; get high word of long argument
	inc	bx
prna4:	cmp	al,'d'			; the number is now in (DX,AX):CX
	mov	ax,dx
	jne	short prnaB		; if it's negative, print a minus
	or	dx,dx			; sign, and then convert it into
	jns	short prnaB		; a positive number
	mov	al,'-'
	int	29h			; print minus sign
	xor	ax,ax
	neg	cx			; make the number positive
	sbb	ax,dx
prnaB:	pop	dx
	xchg	ax,cx			; the argument is now in CX:AX
	call 	prnum			; print the number
	jmp	short prna9

prna5:	cmp	al,'c'			; check for character argument
	jne	short prna6
	getarg	ax,bx
	inc	bx
	jmp	print4			; print character

prna6:	cmp	al,'s'			; check for string argument
	jne	short prna9
	push	si
	getarg	si,bx			; get pointer to string
	inc	bx
	mov	cx,-1			; check for length argument
	or	ah,ah
	jz	short prna7
	getarg	cx,bx			; get maximum length of string
	inc	bx
prna7:	lodsb				; print string
	or	al,al
	jz	short prna8
	int	29h			; print the character
	loop	short prna7
prna8:	pop	si
prna9:	jmp	print1

; Return to caller after cleaning up the stack.

print9:	pop	di
	pop	si
	pleave
	ret

_printf		endp


;
;**************************************************************************
;
; Print a number onto the screen.
; Input:  CX:AX  -  number
;         DL     -  base
; Output: none
; Registers changed: AX, CX, DX
;
prnum		proc	near

	push	bx
	mov	bx,offset dgroup:numbuf + size numbuf - 1
prnum1:	xchg	cx,ax
	mov	dh,al
	mov	al,ah
	xor	ah,ah
	div	dl
	xchg	dh,al			; divide the number in CX:AX by
	div	dl			; the number base
	xchg	al,ch
	div	dl
	xchg	al,cl
	div	dl
	and	ah,0Fh
	add	ah,'0'
	cmp	ah,'9'			; the division remainder gets converted
	jbe	short prnum2		; into an ascii character
	add	ah,7
prnum2:	mov	[bx],ah
	dec	bx
	cmp	bx,offset dgroup:numbuf	; store character into temporary buffer
	jbe	short prnum3
	mov	ah,cl
	mov	cl,ch			; put quotient back into CX:AX
	mov	ch,dh
	or	dh,cl
	or	dh,ah
	or	dh,al			; check if number got completely
	or	dh,dh			; converted
	jnz	short prnum1

prnum3:	inc	bx
	mov	al,[bx]			; print all characters using DOS
	int	29h
	cmp	bx,offset dgroup:numbuf + size numbuf - 1
	jb	short prnum3
	pop	bx
	ret

prnum		endp


;
;**************************************************************************
;
text_end

	end

